@@ -23,11 +23,17 @@
 ####################
 
 import logging
+import ssl
 import os
 import traceback
+from hashlib import sha256, md5
 from bloodhound.ad.utils import CollectionException
 from binascii import unhexlify
-from ldap3 import Server, Connection, NTLM, ALL, SASL, KERBEROS
+from cryptography import x509
+from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives import hashes
+import ldap3
+from ldap3 import Server, Connection, NTLM, ALL, SASL, KERBEROS, Tls
 from ldap3.core.results import RESULT_STRONGER_AUTH_REQUIRED
 from ldap3.operation.bind import bind_operation
 from impacket.krb5.ccache import CCache
@@ -37,6 +43,7 @@ from impacket.krb5.asn1 import AP_REQ, AS_REP, TGS_REQ, Authenticator, TGS_REP,
     Ticket as TicketAsn1, EncTGSRepPart
 from impacket.krb5 import constants
 from impacket.krb5.kerberosv5 import getKerberosTGT, getKerberosTGS, sendReceive
+from impacket.krb5.gssapi import CheckSumField, GSS_C_SEQUENCE_FLAG, GSS_C_REPLAY_FLAG, GSS_C_MUTUAL_FLAG
 import datetime
 from pyasn1.type.univ import noValue
 from impacket.spnego import SPNEGO_NegTokenInit, TypesMech
@@ -46,7 +53,7 @@ Active Directory authentication helper
 """
 class ADAuthentication(object):
     def __init__(self, username='', password='', domain='',
-                 lm_hash='', nt_hash='', aeskey='', kdc=None, auth_method='auto'):
+                 lm_hash='', nt_hash='', aeskey='', kdc=None, auth_method='auto', ldap_channel_binding=False):
         self.username = username
         # Assume user domain and enum domain are same
         self.domain = domain.lower()
@@ -63,6 +70,7 @@ class ADAuthentication(object):
         # KDC for domain of the user - fill with domain first, will be resolved later
         self.userdomain_kdc = self.domain
         self.auth_method = auth_method
+        self.ldap_channel_binding = ldap_channel_binding
 
         # Kerberos
         self.tgt = None
@@ -80,54 +88,103 @@ class ADAuthentication(object):
     def getLDAPConnection(self, hostname='', ip='', baseDN='', protocol='ldaps', gc=False):
         if gc:
             # Global Catalog connection
-            if protocol == 'ldaps':
-                # Ldap SSL
-                server = Server("%s://%s:3269" % (protocol, ip), get_info=ALL)
+            if protocol == 'ldaps' or self.ldap_channel_binding is True:
+                if self.ldap_channel_binding is True:
+                    if not hasattr(ldap3, 'TLS_CHANNEL_BINDING'):
+                        raise Exception("To use LDAP channel binding, install the patched ldap3 module: pip3 install git+https://github.com/ly4k/ldap3 or pip3 install ldap3-bleeding-edge")
+                    logging.debug("Using LDAPS channel binding")
+                    protocol = 'ldaps'
+                    version=ssl.PROTOCOL_TLSv1_2
+                    tls = Tls(validate=ssl.CERT_NONE, version=version, ciphers='ALL:@SECLEVEL=0')
+                    server = Server(
+                        "%s://%s:3269" % (protocol,ip),
+                        use_ssl=True,
+                        get_info=ALL,
+                        tls=tls
+                    )
+                else:
+                    # Ldap SSL (no channel binding)
+                    server = Server("%s://%s:3269" % (protocol, ip), get_info=ALL)
             else:
                 # Plain LDAP
                 server = Server("%s://%s:3268" % (protocol, ip), get_info=ALL)
-        else:
-            server = Server("%s://%s" % (protocol, ip), get_info=ALL)
+        else: # no GC specified
+            if self.ldap_channel_binding is True:
+                if not hasattr(ldap3, 'TLS_CHANNEL_BINDING'):
+                    raise Exception("To use LDAP channel binding, install the patched ldap3 module: pip3 install git+https://github.com/ly4k/ldap3 or pip3 install ldap3-bleeding-edge")
+                logging.debug("Using LDAPS channel binding")
+                protocol = 'ldaps'
+                version=ssl.PROTOCOL_TLSv1_2
+                tls = Tls(validate=ssl.CERT_NONE, version=version, ciphers='ALL:@SECLEVEL=0')
+                server = Server(
+                    "%s://%s" % (protocol,ip),
+                    use_ssl=True,
+                    get_info=ALL,
+                    tls=tls
+                )
+            else: # No LDAP Channel Binding
+                server = Server("%s://%s" % (protocol, ip), get_info=ALL)
         # ldap3 supports auth with the NT hash. LM hash is actually ignored since only NTLMv2 is used.
         if self.nt_hash != '':
-            ldappass = self.lm_hash + ':' + self.nt_hash
+            if self.lm_hash != '':
+                ldappass = self.lm_hash + ':' + self.nt_hash
+            else:
+                # ldap3 requires a 32-character long string for LM hash in order to use the NT hash
+                ldappass = 'aad3b435b51404eeaad3b435b51404ee:' + self.nt_hash
         else:
             ldappass = self.password
         ldaplogin = '%s\\%s' % (self.userdomain, self.username)
-        conn = Connection(server, user=ldaplogin, auto_referrals=False, password=ldappass, authentication=NTLM, receive_timeout=60, auto_range=True)
+
         bound = False
         if self.tgt is not None and self.auth_method in ('kerberos', 'auto'):
-            conn = Connection(server, user=ldaplogin, auto_referrals=False, password=ldappass, authentication=SASL, sasl_mechanism=KERBEROS)
+            conn = Connection(server, user=ldaplogin, auto_referrals=False, password=ldappass, authentication=SASL, sasl_mechanism=KERBEROS, receive_timeout=60, auto_range=True)
             logging.debug('Authenticating to LDAP server with Kerberos')
             try:
                 bound = self.ldap_kerberos(conn, hostname)
             except Exception as exc:
                 if self.auth_method == 'auto':
                     logging.debug(traceback.format_exc())
-                    logging.info('Kerberos auth to LDAP failed, trying NTLM')
+                    logging.warning('Kerberos auth to LDAP failed, trying NTLM')
                     bound = False
                 else:
-                    logging.debug('Kerberos auth to LDAP failed, no authentication methods left')
-
+                    logging.debug(traceback.format_exc())
+                    logging.critical('Kerberos auth to LDAP failed, no authentication methods left')
+                    raise CollectionException('Could not authenticate to LDAP. Check your credentials and LDAP server requirements.')
         if not bound:
-            conn = Connection(server, user=ldaplogin, auto_referrals=False, password=ldappass, authentication=NTLM)
+            conn = Connection(server, user=ldaplogin, auto_referrals=False, password=ldappass, authentication=NTLM, receive_timeout=60, auto_range=True)
             logging.debug('Authenticating to LDAP server with NTLM')
+            if self.ldap_channel_binding:
+                from ldap3 import TLS_CHANNEL_BINDING
+                logging.debug("Using LDAPS channel binding")
+                protocol = 'ldaps'
+                channel_binding = {"channel_binding": TLS_CHANNEL_BINDING}
+                conn = Connection(server, user=ldaplogin, password=ldappass, authentication=NTLM, auto_referrals=False, receive_timeout=60, auto_range=True, **channel_binding)
+            else:
+                conn = Connection(server, user=ldaplogin, auto_referrals=False, password=ldappass, authentication=NTLM, receive_timeout=60, auto_range=True)
             bound = conn.bind()
 
         if not bound:
             result = conn.result
+            if result['result'] == RESULT_STRONGER_AUTH_REQUIRED and protocol == 'ldaps':
+                logging.warning('LDAP Authentication is refused because LDAP Channel Binding is likely enabled. '
+                                'Trying to connect using LDAP Channel Binding')
+                self.ldap_channel_binding = True
+                return self.getLDAPConnection(hostname, ip, baseDN, 'ldaps')
             if result['result'] == RESULT_STRONGER_AUTH_REQUIRED and protocol == 'ldap':
                 logging.warning('LDAP Authentication is refused because LDAP signing is enabled. '
                                 'Trying to connect over LDAPS instead...')
                 return self.getLDAPConnection(hostname, ip, baseDN, 'ldaps')
             else:
-                logging.error('Failure to authenticate with LDAP! Error %s' % result['message'])
+                logging.error('Failure to authenticate with LDAP! Error %s : Code: %s' % (result['message'], result['result']))
                 raise CollectionException('Could not authenticate to LDAP. Check your credentials and LDAP server requirements.')
         return conn
 
     def ldap_kerberos(self, connection, hostname):
         # Hackery to authenticate with ldap3 using impacket Kerberos stack
 
+        # Open ldap3 socket because we need it
+        connection.open(read_server_info=False)
+
         username = Principal(self.username, type=constants.PrincipalNameType.NT_PRINCIPAL.value)
         servername = Principal('ldap/%s' % hostname, type=constants.PrincipalNameType.NT_SRV_INST.value)
         tgs, cipher, _, sessionkey = getKerberosTGS(servername, self.domain, self.kdc,
@@ -162,6 +219,57 @@ class ADAuthentication(object):
         authenticator['cusec'] = now.microsecond
         authenticator['ctime'] = KerberosTime.to_asn1(now)
 
+        peercert = None
+        bindings = None
+        try:
+            peercert = connection.socket.getpeercert(binary_form=True)
+        except AttributeError:
+            # No TLS, skip
+            pass
+        if peercert:
+            # Do TLS channel binding
+            # The logic here is heavly inspired by "msldap", "minikerberos" and "asysocks" projects by @skelsec.
+            # Adapted from ldap3 contributions by ThePirateWhoSmellsOfSunflowers
+            peer_certificate = x509.load_der_x509_certificate(peercert, default_backend())
+            peer_certificate_hash_algorithm = peer_certificate.signature_hash_algorithm
+
+            # RFC 5929 section 4.1 hashes list
+            rfc5929_hashes_list = (hashes.MD5, hashes.SHA1)
+
+            # section 4.1 hash function selection
+            if isinstance(peer_certificate_hash_algorithm, rfc5929_hashes_list):
+                digest = hashes.Hash(hashes.SHA256(), default_backend())
+            else:
+                digest = hashes.Hash(peer_certificate_hash_algorithm, default_backend())
+            digest.update(peercert)
+            peer_certificate_digest = digest.finalize()
+
+            # https://datatracker.ietf.org/doc/html/rfc2744#section-3.11
+            channel_binding_struct = bytes()
+            initiator_address = b'\x00'*8
+            acceptor_address = b'\x00'*8
+
+            # https://datatracker.ietf.org/doc/html/rfc5929#section-4
+            application_data_raw = b'tls-server-end-point:' + peer_certificate_digest
+            len_application_data = len(application_data_raw).to_bytes(4, byteorder='little', signed = False)
+            application_data = len_application_data
+            application_data += application_data_raw
+            channel_binding_struct += initiator_address
+            channel_binding_struct += acceptor_address
+            channel_binding_struct += application_data
+            bindings = md5(channel_binding_struct).digest()
+
+        # Add checksum to authenticator
+        authenticator['cksum'] = noValue
+        authenticator['cksum']['cksumtype'] = 0x8003
+
+        chkField = CheckSumField()
+        chkField['Lgth'] = 16
+        if bindings:
+            chkField['Bnd'] = bindings
+        chkField['Flags'] = 0
+        authenticator['cksum']['checksum'] = chkField.getData()
+
         encodedAuthenticator = encoder.encode(authenticator)
 
         # Key Usage 11
@@ -177,7 +285,6 @@ class ADAuthentication(object):
         blob['MechToken'] = encoder.encode(apReq)
 
         # From here back to ldap3
-        connection.open(read_server_info=False)
         request = bind_operation(connection.version, SASL, None, None, connection.sasl_mechanism, blob.getData())
         response = connection.post_send_single_response(connection.send('bindRequest', request, None))[0]
         connection.result = response
